# Grabbable Modifier Keys

This module implements grabbable modifier keys, such that a press or release of
a modifier key may trigger an action. 

## Usage

Ensure this module is loaded. Then use `SWM/GMK:SET-MODIFIER-ACTIONS` to set the
actions for a given modifier. E.g. to make super toggle the modline on
press/release you could do the following: 

```
(flet ((toggler (c s)
         (declare (ignore c s))
         (let* ((s (stumpwm:current-screen))
                (g (stumpwm:current-group s))
                (h (stumpwm:current-head g)))
           (stumpwm:toggle-mode-line s h))))
  (set-modifier-actions :mod4 #'toggler #'toggler))
```

## Quirks

Modifier key presses do not follow the normal key binding and command invocation
of StumpWM. Rather they expose the key handler parameters code and state to the
user, and expect the user to do whatever legwork is neccessary based on those
parameters. 

Additionally, the key press will not have the modifier set in its state, but on
key release the modifier will be set in the state mask. E.g. from the above
example if only super is pressed with no other modifiers then the state will be
zero on press and 64 on release.

If a key is defined in the top map that uses the modifier, then a press to that
key will prevent the firing of the release action. E.g. from our above example,
if `super-p` is bound then pressing super will toggle the mode line, pressing p
will fire whatever command it is bound to, and releasing super will ***not***
fire the release action.
